@@ -1,8 +1,8 @@
 # -*- coding: iso-8859-1 -*-
 # -----------------------------------------------------------------------
-# directory.py - Directory handling
+# Directory handling
 # -----------------------------------------------------------------------
-# $Id: directory.py 10562 2008-03-22 21:57:15Z duncan $
+# $Id: directory.py 11557 2009-05-24 15:19:31Z duncan $
 #
 # Notes:
 # Todo:
@@ -28,6 +28,9 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Directory handling
+"""
 
 import os
 import traceback
@@ -37,11 +40,11 @@ import copy
 import rc
 from pprint import pformat
 
-import util.mediainfo as mediainfo
-import kaa.metadata as mmpython
+import kaa.metadata
 
 import config
 import util
+import util.mediainfo as mediainfo
 
 import menu
 import skin
@@ -54,44 +57,44 @@ from playlist import Playlist
 from event import *
 from gui import InputBox, AlertBox, ProgressBox
 
-all_variables = [('DIRECTORY_SORT_BY_DATE', _('Directory Sort By Date'),
-                  _('Sort directory by date and not by name.'), False),
 
-                 ('DIRECTORY_AUTOPLAY_SINGLE_ITEM', _('Directory Autoplay Single Item'),
-                  _('Don\'t show directory if only one item exists and auto-select ' \
-                    'the item.'), False),
+all_variables = [
+    ('DIRECTORY_SORT_BY_DATE', _('Directory Sort By Date'),
+        _('Sort directory by date and not by name.'), False),
 
-                 ('DIRECTORY_FORCE_SKIN_LAYOUT', _('Force Skin Layout'),
-                  _('Force skin to a specific layout. This option doesn\'t work with ' \
-                    'all skins and the result may differ based on the skin.'), False),
+    ('DIRECTORY_AUTOPLAY_SINGLE_ITEM', _('Directory Autoplay Single Item'),
+        _('Don\'t show directory if only one item exists and auto-select the item.'), False),
 
-                 ('DIRECTORY_SMART_SORT', _('Directory Smart Sort'),
-                  _('Use a smarter way to sort the items.'), False),
+    ('DIRECTORY_FORCE_SKIN_LAYOUT', _('Force Skin Layout'),
+        _('Force skin to a specific layout. This option doesn\'t work with ' \
+        'all skins and the result may differ based on the skin.'), False),
 
-                 ('DIRECTORY_SMART_NAMES', _('Directory Smart Names'),
-                  _('Use a smarter way to abbreviate the name of the items.'), True),
+    ('DIRECTORY_SMART_SORT', _('Directory Smart Sort'),
+        _('Use a smarter way to sort the items.'), False),
 
-                 ('DIRECTORY_USE_MEDIAID_TAG_NAMES', _('Use MediaID Tag Names'),
-                  _('Use the names from the media files tags as display name.'), False),
+    ('DIRECTORY_SMART_NAMES', _('Directory Smart Names'),
+        _('Use a smarter way to abbreviate the name of the items.'), True),
 
-                 ('DIRECTORY_REVERSE_SORT', _('Directory Reverse Sort'),
-                  _('Show the items in the list in reverse order.'), False),
+    ('DIRECTORY_USE_MEDIAID_TAG_NAMES', _('Use MediaID Tag Names'),
+        _('Use the names from the media files tags as display name.'), False),
 
-                 ('DIRECTORY_AUDIO_FORMAT_STRING', '', '', False),
+    ('DIRECTORY_REVERSE_SORT', _('Directory Reverse Sort'),
+        _('Show the items in the list in reverse order.'), False),
 
-                 ('DIRECTORY_CREATE_PLAYLIST', _('Directory Create Playlist'),
-                  _('Handle the directory as playlist. After one file is played, the next '\
-                    'one will be started.'), True) ,
+    ('DIRECTORY_AUDIO_FORMAT_STRING', '', '', False),
 
-                 ('DIRECTORY_ADD_PLAYLIST_FILES', _('Directory Add Playlist Files'),
-                  _('Add playlist files to the list of items'), True) ,
+    ('DIRECTORY_CREATE_PLAYLIST', _('Directory Create Playlist'),
+        _('Handle the directory as playlist. After one file is played, the next one will be started.'), True),
 
-                 ('DIRECTORY_ADD_RANDOM_PLAYLIST', _('Directory Add Random Playlist'),
-                  _('Add an item for a random playlist'), True) ,
+    ('DIRECTORY_ADD_PLAYLIST_FILES', _('Directory Add Playlist Files'),
+        _('Add playlist files to the list of items'), True) ,
 
-                 ('DIRECTORY_AUTOPLAY_ITEMS', _('Directory Autoplay Items'),
-                  _('Autoplay the whole directory (as playlist) when it contains only '\
-                    'files and no directories' ), True)]
+    ('DIRECTORY_ADD_RANDOM_PLAYLIST', _('Directory Add Random Playlist'),
+        _('Add an item for a random playlist'), True) ,
+
+    ('DIRECTORY_AUTOPLAY_ITEMS', _('Directory Autoplay Items'),
+        _('Autoplay the whole directory (as playlist) when it contains only files and no directories'), True)
+]
 
 
 class DirItem(Playlist):
@@ -99,6 +102,8 @@ class DirItem(Playlist):
     class for handling directories
     """
     def __init__(self, directory, parent, name='', display_type=None, add_args=None, create_metainfo=True):
+        if directory.startswith('/misc'):
+            _debug_('DirItem.__init__(directory=%r, parent=%r, name=%r, display_type=%r, add_args=%r, create_metainfo=%r)' % (directory, parent, name, display_type, add_args, create_metainfo), 2)
         self.autovars = [ ('num_dir_items', 0), ('show_all_items', False) ]
         Playlist.__init__(self, parent=parent, display_type=display_type)
         self.type = 'dir'
@@ -115,7 +120,7 @@ class DirItem(Playlist):
 
         #FIXME This should be done in the cache create
         if not self.image:
-            mminfo = mmpython.parse(directory)
+            mminfo = kaa.metadata.parse(directory)
             if mminfo:
                 if mminfo['image']:
                     self.image = mminfo['image']
@@ -205,16 +210,15 @@ class DirItem(Playlist):
 
     def __repr__(self):
         if hasattr(self, 'name'):
-            s = '<%s: %r>' % (self.name, self.__class__)
-        else:
-            s = '<%r>' % (self.__class__)
-        return s
+            return '<%s: %r>' % (self.name, self.__class__)
+        return '<%r>' % (self.__class__)
 
 
     def set_fxd_file(self, file):
         """
         Set self.folder_fxd and parse it
         """
+        _debug_('set_fxd_file(file=%r)' % (file,), 2)
         self.folder_fxd = file
         if self.folder_fxd and vfs.isfile(self.folder_fxd):
             if self.display_type == 'tv':
@@ -225,7 +229,11 @@ class DirItem(Playlist):
                 parser.set_handler('skin', self.read_folder_fxd)
                 parser.parse()
             except:
-                print "fxd file %s corrupt" % self.folder_fxd
+                _debug_('fxd file %r corrupt, removed' % (self.folder_fxd,), DWARNING)
+                try:
+                    os.remove(self.folder_fxd)
+                except OSError, why:
+                    _debug_('fxd file %r not removed' % (self.folder_fxd,), DINFO)
                 traceback.print_exc()
 
 
@@ -243,6 +251,7 @@ class DirItem(Playlist):
                 </folder>
             </freevo>
         """
+        _debug_('read_folder_fxd(fxd=%r, node=%r)' % (fxd, node), 2)
         if node.name == 'skin':
             self.skin_fxd = self.folder_fxd
             return
@@ -255,12 +264,11 @@ class DirItem(Playlist):
             self.image = os.path.join(self.dir, image)
 
         # parse <info> tag
-        fxd.parse_info(fxd.get_children(node, 'info', 1), self,
-                       {'description': 'content', 'content': 'content' })
+        fxd.parse_info(fxd.get_children(node, 'info', 1), self, {'description': 'content', 'content': 'content' })
 
         for child in fxd.get_children(node, 'setvar', 1):
             # <setvar name="directory_smart_sort" val="1"/>
-            for v,n,d, type_list in self.all_variables:
+            for v, n, d, type_list in self.all_variables:
                 if child.attrs[('', 'name')].upper() == v.upper():
                     if type_list:
                         if int(child.attrs[('', 'val')]):
@@ -279,7 +287,8 @@ class DirItem(Playlist):
         """
         return if this variable to be saved is a type_list
         """
-        for v,n,d, type_list in self.all_variables:
+        _debug_('__is_type_list_var__(var=%r)' % (var,), 2)
+        for v, n, d, type_list in self.all_variables:
             if v == var:
                 return type_list
         return False
@@ -289,6 +298,7 @@ class DirItem(Playlist):
         """
         callback to save the modified fxd file
         """
+        _debug_('write_folder_fxd(fxd=%r, node=%r)' % (fxd, node), 2)
         # remove old setvar
         for child in copy.copy(node.children):
             if child.name == 'setvar':
@@ -327,7 +337,7 @@ class DirItem(Playlist):
                 display_type = 'video'
             return self['num_%s_items' % display_type]
 
-        if key in ( 'freespace', 'totalspace' ):
+        if key in ('freespace', 'totalspace'):
             if self.media:
                 return None
 
@@ -341,6 +351,7 @@ class DirItem(Playlist):
 
     # eventhandler for this item
     def eventhandler(self, event, menuw=None):
+        _debug_('eventhandler(event=%r, menuw=%r)' % (event, menuw), 2)
         if event == DIRECTORY_CHANGE_DISPLAY_TYPE and menuw.menustack[-1] == self.menu:
             possible_display_types = [ ]
 
@@ -390,8 +401,7 @@ class DirItem(Playlist):
             if var == 'num_%s_timestamp' % name:
                 break
         else:
-            self.autovars += [ ( 'num_%s_timestamp' % name, 0 ),
-                               ( 'num_%s_items' % name, 0 ) ]
+            self.autovars += [ ('num_%s_timestamp' % name, 0), ('num_%s_items' % name, 0) ]
 
         try:
             timestamp = os.stat(self.dir)[stat.ST_MTIME]
@@ -402,9 +412,7 @@ class DirItem(Playlist):
 
         if not num_timestamp or num_timestamp < timestamp:
             _debug_('create metainfo for %s', self.dir)
-            need_umount = False
             if self.media:
-                need_umount = not self.media.is_mounted()
                 self.media.mount()
 
             num_dir_items  = 0
@@ -421,13 +429,11 @@ class DirItem(Playlist):
                     num_dir_items += 1
 
             # store info
-            if num_play_items != self['num_%s_items' % name]:
-                self['num_%s_items' % name] = num_play_items
-            if self['num_dir_items'] != num_dir_items:
-                self['num_dir_items'] = num_dir_items
+            self['num_dir_items'] = num_dir_items
+            self['num_%s_items' % name] = num_play_items
             self['num_%s_timestamp' % name] = timestamp
 
-            if need_umount:
+            if self.media:
                 self.media.umount()
 
 
@@ -439,6 +445,7 @@ class DirItem(Playlist):
         """
         return a list of actions for this item
         """
+        _debug_('actions()', 2)
         if self.media:
             self.media.mount()
 
@@ -446,7 +453,7 @@ class DirItem(Playlist):
         if self.display_type == 'tv':
             display_type = 'video'
 
-        items = [ ( self.cwd, _('Browse directory')) ]
+        items = [ (self.cwd, _('Browse directory')) ]
 
         if self['num_%s_items' % display_type]:
             items.append((self.play, _('Play all files in directory')))
@@ -456,9 +463,10 @@ class DirItem(Playlist):
 
         if self['num_%s_items' % display_type]:
             items.append((self.play_random, _('Random play all items')))
+
         if self['num_dir_items']:
-            items += [ (self.play_random_recursive, _('Recursive random play all items')),
-                       (self.play_recursive, _('Recursive play all items')) ]
+            items.append((self.play_random_recursive, _('Recursive random play all items')))
+            items.append((self.play_recursive, _('Recursive play all items')))
 
         items.append((self.configure, _('Configure directory'), 'configure'))
 
@@ -476,6 +484,7 @@ class DirItem(Playlist):
         """
         browse directory
         """
+        _debug_('cwd(arg=%r, menuw=%r)' % (arg, menuw), 2)
         self.check_password_and_build(arg=None, menuw=menuw)
 
 
@@ -483,6 +492,7 @@ class DirItem(Playlist):
         """
         play directory
         """
+        _debug_('%s.play(arg=%r, menuw=%r)' % (self.__module__, arg, menuw))
         if arg == 'next':
             Playlist.play(self, arg=arg, menuw=menuw)
         else:
@@ -493,6 +503,7 @@ class DirItem(Playlist):
         """
         play in random order
         """
+        _debug_('play_random(arg=%r, menuw=%r)' % (arg, menuw))
         self.check_password_and_build(arg='playlist:random', menuw=menuw)
 
 
@@ -500,6 +511,7 @@ class DirItem(Playlist):
         """
         play recursive
         """
+        _debug_('play_recursive(arg=%r, menuw=%r)' % (arg, menuw))
         self.check_password_and_build(arg='playlist:recursive', menuw=menuw)
 
 
@@ -507,6 +519,7 @@ class DirItem(Playlist):
         """
         play recursive in random order
         """
+        _debug_('play_random_recursive(arg=%r, menuw=%r)' % (arg, menuw))
         self.check_password_and_build(arg='playlist:random_recursive', menuw=menuw)
 
 
@@ -514,23 +527,37 @@ class DirItem(Playlist):
         """
         password checker
         """
+        _debug_('check_password_and_build(arg=%r, menuw=%r)' % (arg, menuw))
         if not self.menuw:
             self.menuw = menuw
 
-        # are we on a ROM_DRIVE and have to mount it first?
-        for media in config.REMOVABLE_MEDIA:
-            if self.dir.find(media.mountdir) == 0:
-                util.mount(self.dir)
-                self.media = media
+        if self.media:
+            self.media.mount()
 
-        if vfs.isfile(self.dir + '/.password'):
-            print 'password protected dir'
-            self.arg   = arg
+        if vfs.isfile(os.path.join(self.dir + '.password')):
+            _debug_('password protected dir', DINFO)
+            try:
+                pwfile = vfs.open(os.path.join(self.dir + '.password'))
+                line = pwfile.readline()
+                pwfile.close()
+            except IOError, e:
+                _debug_( 'error reading password file for %s : %s' % (self.dir,str(e)), DWARNING )
+                pb = AlertBox(text=(_('Error reading password file:') + str(e)))
+                pb.show()
+                if self.media:
+                    self.media.umount()
+                return
+
+            if self.media:
+                self.media.umount()
+
+            self.arg   = (arg, line.strip())
             self.menuw = menuw
-            pb = InputBox(text=_('Enter Password'), handler=self.pass_cmp_cb,
-                          type='password')
+            pb = InputBox(text=_('Enter Password'), handler=self.pass_cmp_cb, type='password')
             pb.show()
         else:
+            if self.media:
+                self.media.umount()
             self.build(arg=arg, menuw=menuw)
 
 
@@ -539,18 +566,10 @@ class DirItem(Playlist):
         read the contents of self.dir/.passwd and compare to word
         callback for check_password_and_build
         """
-        try:
-            pwfile = vfs.open(self.dir + '/.password')
-            line = pwfile.readline()
-        except IOError, e:
-            print 'error %d (%s) reading password file for %s' % \
-                  (e.errno, e.strerror, self.dir)
-            return
-
-        pwfile.close()
-        password = line.strip()
+        _debug_('pass_cmp_cb(word=%r)' % (word,), 2)
+        (arg,password) = self.arg
         if word == password:
-            self.build(self.arg, self.menuw)
+            self.build(arg, self.menuw)
         else:
             pb = AlertBox(text=_('Password incorrect'))
             pb.show()
@@ -558,18 +577,24 @@ class DirItem(Playlist):
 
 
     def build(self, arg=None, menuw=None):
+        if self.media:
+            self.media.mount()
+        self.__build(arg,menuw)
+        if self.media:
+            self.media.umount()
+
+
+    def __build(self, arg=None, menuw=None):
         """
         build the items for the directory
         """
+        _debug_('build(arg=%r, menuw=%r)' % (arg, menuw), 2)
         self.menuw      = menuw
         self.playlist   = []
         self.play_items = []
         self.dir_items  = []
         self.pl_items   = []
 
-        if self.media:
-            self.media.mount()
-
         if hasattr(self, '__dirwatcher_last_time__'):
             del self.__dirwatcher_last_time__
 
@@ -587,7 +612,7 @@ class DirItem(Playlist):
             if hasattr(self.menu, 'skin_force_text_view'):
                 del self.menu.skin_force_text_view
         elif not os.path.exists(self.dir):
-            AlertBox(text=_('Directory does not exist')).show()
+            AlertBox(text=_('Directory %r does not exist' % (self.dir))).show()
             return
 
         display_type = self.display_type
@@ -597,46 +622,42 @@ class DirItem(Playlist):
         if arg and arg.startswith('playlist:'):
             if arg.endswith(':random'):
                 Playlist(playlist = [ (self.dir, 0) ], parent = self,
-                         display_type=display_type, random=True).play(menuw=menuw)
+                    display_type=display_type, random=True).play(menuw=menuw)
             elif arg.endswith(':recursive'):
                 Playlist(playlist = [ (self.dir, 1) ], parent = self,
-                         display_type=display_type, random=False).play(menuw=menuw)
+                    display_type=display_type, random=False).play(menuw=menuw)
             elif arg.endswith(':random_recursive'):
                 Playlist(playlist = [ (self.dir, 1) ], parent = self,
-                         display_type=display_type, random=True).play(menuw=menuw)
+                    display_type=display_type, random=True).play(menuw=menuw)
             return
 
-        if config.OSD_BUSYICON_TIMER:
-            osd.get_singleton().busyicon.wait(config.OSD_BUSYICON_TIMER[0])
+        osd.get_singleton().busyicon.wait(config.OSD_BUSYICON_TIMER[0])
 
         files       = vfs.listdir(self.dir, include_overlay=True)
         num_changes = mediainfo.check_cache(self.dir)
 
         pop = None
-        callback=None
+        callback = None
         if skin.active():
             if (num_changes > 10) or (num_changes and self.media):
                 if self.media:
                     pop = ProgressBox(text=_('Scanning disc, be patient...'), full=num_changes)
                 else:
-                    pop = ProgressBox(text=_('Scanning directory, be patient...'),
-                                      full=num_changes)
+                    pop = ProgressBox(text=_('Scanning directory, be patient...'), full=num_changes)
                 pop.show()
                 callback=pop.tick
 
-
-        elif config.OSD_BUSYICON_TIMER and len(files) > config.OSD_BUSYICON_TIMER[1]:
+        elif len(files) > config.OSD_BUSYICON_TIMER[1]:
             # many files, just show the busy icon now
             osd.get_singleton().busyicon.wait(0)
 
-
         if num_changes > 0:
             mediainfo.cache_dir(self.dir, callback=callback)
 
-
         #
         # build items
         #
+
         # build play_items, pl_items and dir_items
         for p in plugin.mimetype(display_type):
             for i in p.get(self, files):
@@ -650,7 +671,7 @@ class DirItem(Playlist):
         # normal DirItems
         for filename in files:
             if os.path.isdir(filename):
-                d = DirItem(filename, self, display_type = self.display_type)
+                d = DirItem(filename, self, display_type=self.display_type)
                 self.dir_items.append(d)
 
         # remove same beginning from all play_items
@@ -675,7 +696,7 @@ class DirItem(Playlist):
 
         # sort directories
         if self.DIRECTORY_SMART_SORT:
-            self.dir_items.sort(lambda l, o: util.smartsort(l.dir,o.dir))
+            self.dir_items.sort(lambda l, o: util.smartsort(l.dir, o.dir))
         else:
             self.dir_items.sort(lambda l, o: cmp(l.dir.upper(), o.dir.upper()))
 
@@ -684,19 +705,15 @@ class DirItem(Playlist):
 
         # sort normal items
         if self.DIRECTORY_SORT_BY_DATE:
-            self.play_items.sort(lambda l, o: cmp(l.sort('date').upper(),
-                                                  o.sort('date').upper()))
+            self.play_items.sort(lambda l, o: cmp(l.sort('date').upper(), o.sort('date').upper()))
         elif self['%s_advanced_sort' % display_type]:
-            self.play_items.sort(lambda l, o: cmp(l.sort('advanced').upper(),
-                                                  o.sort('advanced').upper()))
+            self.play_items.sort(lambda l, o: cmp(l.sort('advanced').upper(), o.sort('advanced').upper()))
         else:
             self.play_items.sort(lambda l, o: cmp(l.sort().upper(), o.sort().upper()))
 
-        if self['num_dir_items'] != len(self.dir_items):
-            self['num_dir_items'] = len(self.dir_items)
+        self['num_dir_items'] = len(self.dir_items)
 
-        if self['num_%s_items' % display_type] != len(self.play_items) + len(self.pl_items):
-            self['num_%s_items' % display_type] = len(self.play_items) + len(self.pl_items)
+        self['num_%s_items' % display_type] = len(self.play_items) + len(self.pl_items)
 
         if self.DIRECTORY_REVERSE_SORT:
             self.dir_items.reverse()
@@ -712,29 +729,22 @@ class DirItem(Playlist):
         if not self.display_type or self.display_type in self.DIRECTORY_CREATE_PLAYLIST:
             self.playlist = self.play_items
 
-
         # build a list of all items
         items = self.dir_items + self.pl_items + self.play_items
 
         # random playlist (only active for audio)
-        if self.display_type and self.display_type in self.DIRECTORY_ADD_RANDOM_PLAYLIST \
-               and len(self.play_items) > 1:
+        if self.display_type and self.display_type in self.DIRECTORY_ADD_RANDOM_PLAYLIST and len(self.play_items) > 1:
             pl = Playlist(_('Random playlist'), self.play_items, self, random=True)
             pl.autoplay = True
             items = [ pl ] + items
 
         if pop:
             pop.destroy()
-            # closing the poup will rebuild the menu which may umount
-            # the drive
-            if self.media:
-                self.media.mount()
 
-        if config.OSD_BUSYICON_TIMER:
-            # stop the timer. If the icons is drawn, it will stay there
-            # until the osd is redrawn, if not, we don't need it to pop
-            # up the next milliseconds
-            osd.get_singleton().busyicon.stop()
+        # stop the timer. If the icons is drawn, it will stay there
+        # until the osd is redrawn, if not, we don't need it to pop
+        # up the next milliseconds
+        osd.get_singleton().busyicon.stop()
 
 
         #
@@ -765,9 +775,7 @@ class DirItem(Playlist):
                     self.menuw.init_page()
                 self.menuw.refresh()
 
-
-        elif len(items) == 1 and items[0].actions() and \
-                 self.DIRECTORY_AUTOPLAY_SINGLE_ITEM:
+        elif len(items) == 1 and items[0].actions() and self.DIRECTORY_AUTOPLAY_SINGLE_ITEM:
             # autoplay
             items[0].actions()[0][0](menuw=menuw)
 
@@ -779,25 +787,23 @@ class DirItem(Playlist):
         else:
             # normal menu build
             item_menu = menu.Menu(self.name, items, reload_func=self.reload,
-                                  item_types = self.skin_display_type,
-                                  force_skin_layout = self.DIRECTORY_FORCE_SKIN_LAYOUT)
+                item_types=self.skin_display_type, force_skin_layout=self.DIRECTORY_FORCE_SKIN_LAYOUT)
 
             if self.skin_fxd:
                 item_menu.skin_settings = skin.load(self.skin_fxd)
 
             menuw.pushmenu(item_menu)
 
-            dirwatcher.cwd(menuw, self, item_menu, self.dir)
+            dirwatcher.cwd(menuw, self, item_menu, self.dir, self.media)
             self.menu  = item_menu
             self.menuw = menuw
 
 
-
     def reload(self):
         """
         called when we return to this menu
         """
-        dirwatcher.cwd(self.menuw, self, self.menu, self.dir)
+        dirwatcher.cwd(self.menuw, self, self.menu, self.dir, self.media)
         dirwatcher.scan()
 
         # we changed the menu, don't build a new one
@@ -813,6 +819,7 @@ class DirItem(Playlist):
         """
         return name for the configure menu
         """
+        _debug_('configure_set_name(name=%r)' % (name,), 2)
         if name in self.modified_vars:
             if name == 'FORCE_SKIN_LAYOUT':
                 return 'ICON_RIGHT_%s_%s' % (str(getattr(self, arg)),
@@ -833,6 +840,7 @@ class DirItem(Playlist):
         Update the variable in arg and change the menu. This function is used by
         'configure'
         """
+        _debug_('configure_set_var(arg=%r, menuw=%r)' % (arg, menuw), 2)
 
         # get current value, None == no special settings
         if arg in self.modified_vars:
@@ -895,8 +903,7 @@ class DirItem(Playlist):
             traceback.print_exc()
 
         # rebuild menu
-        menuw.menustack[-1].choices[menuw.menustack[-1].choices.\
-                                    index(menuw.menustack[-1].selected)] = item
+        menuw.menustack[-1].choices[menuw.menustack[-1].choices.index(menuw.menustack[-1].selected)] = item
         menuw.menustack[-1].selected = item
         menuw.refresh(reload=1)
 
@@ -905,6 +912,7 @@ class DirItem(Playlist):
         """
         change display type from specific to all
         """
+        _debug_('configure_set_display_type(arg=%r, menuw=%r)' % (arg, menuw), 2)
         if self.display_type:
             self['show_all_items'] = True
             self.display_type = None
@@ -919,8 +927,7 @@ class DirItem(Playlist):
         item.name = item.name[:item.name.find(u'\t')]  + name
 
         # rebuild menu
-        menuw.menustack[-1].choices[menuw.menustack[-1].choices.\
-                                    index(menuw.menustack[-1].selected)] = item
+        menuw.menustack[-1].choices[menuw.menustack[-1].choices.index(menuw.menustack[-1].selected)] = item
         menuw.menustack[-1].selected = item
         menuw.refresh(reload=1)
 
@@ -929,11 +936,12 @@ class DirItem(Playlist):
         """
         show the configure dialog for folder specific settings in folder.fxd
         """
+        _debug_('configure(arg=%r, menuw=%r)' % (arg, menuw), 2)
         items = []
         for i, name, descr, type_list in self.all_variables:
             if name == '':
                 continue
-            name += '\t'  + self.configure_set_name(i)
+            name += '\t' + self.configure_set_name(i)
             mi = menu.MenuItem(name, self.configure_set_var, i)
             mi.description = descr
             items.append(mi)
@@ -944,8 +952,7 @@ class DirItem(Playlist):
             else:
                 name = u'\tICON_RIGHT_ON_' + _('on')
 
-            mi = menu.MenuItem(_('Show all kinds of items') + name,
-                               self.configure_set_display_type)
+            mi = menu.MenuItem(_('Show all kinds of items') + name, self.configure_set_display_type)
             mi.description = _('Show video, audio and image items in this directory')
             items.append(mi)
 
@@ -965,12 +972,14 @@ class Dirwatcher(plugin.DaemonPlugin):
     """
 
     def __init__(self):
+        _debug_('Dirwatcher.__init__()', 2)
         plugin.DaemonPlugin.__init__(self)
         self.item          = None
         self.menuw         = None
         self.item_menu     = None
         self.dir           = None
         self.files         = None
+        self.media         = None
         self.poll_interval = 100 # 1 sec
 
         plugin.register(self, 'Dirwatcher')
@@ -981,9 +990,10 @@ class Dirwatcher(plugin.DaemonPlugin):
         Get listing of overlay dir for change checking.
         Do not count *.cache, directories and *.raw (except videofiles.raw)
         """
+        _debug_('listoverlay()', 2)
         if not os.path.isdir(vfs.getoverlay(self.dir)):
             # dir does not exist. Strange, there should be at least an
-            # mmpython cache file in there
+            # kaa.metadata cache file in there
             return []
         ret = []
         for f in os.listdir(vfs.getoverlay(self.dir)):
@@ -991,17 +1001,19 @@ class Dirwatcher(plugin.DaemonPlugin):
                 if f[f[:-4].rfind('.')+1:-4].lower() in config.VIDEO_SUFFIX:
                     ret.append(f)
             else:
-                if not f.endswith('.cache')  and not \
-                   os.path.isdir(os.path.join(self.dir, f)):
+                if not f.endswith('.cache')  and not os.path.isdir(os.path.join(self.dir, f)):
                     ret.append(f)
         return ret
 
 
-    def cwd(self, menuw, item, item_menu, dir):
+    def cwd(self, menuw, item, item_menu, dir, media=None):
+        _debug_('cwd(menuw=%r, item=%r, item_menu=%r, dir=%r, media=%r)' % (menuw, item, item_menu, dir, media), 2)
         self.menuw     = menuw
         self.item      = item
         self.item_menu = item_menu
         self.dir       = dir
+        if self.media and self.media != media:
+            self.media.umount()
         try:
             self.last_time = item.__dirwatcher_last_time__
             self.files     = item.__dirwatcher_last_files__
@@ -1010,6 +1022,9 @@ class Dirwatcher(plugin.DaemonPlugin):
             self.files     = self.listoverlay()
             self.item.__dirwatcher_last_time__  = self.last_time
             self.item.__dirwatcher_last_files__ = self.files
+        if media and self.media != media:
+            media.mount()
+        self.media = media
 
 
     def scan(self):
@@ -1020,7 +1035,7 @@ class Dirwatcher(plugin.DaemonPlugin):
                 return True
         except (OSError, IOError):
             # the directory is gone
-            _debug_('Dirwatcher: unable to read directory %s' % self.dir,1)
+            _debug_('Dirwatcher: unable to read directory %s' % self.dir)
 
             # send EXIT to go one menu up:
             rc.post_event(MENU_BACK_ONE_MENU)
@@ -1054,8 +1069,7 @@ class Dirwatcher(plugin.DaemonPlugin):
 
 
     def poll(self):
-        if self.dir and self.menuw and \
-               self.menuw.menustack[-1] == self.item_menu and not rc.app():
+        if self.dir and self.menuw and self.menuw.menustack[-1] == self.item_menu and not rc.app():
             self.scan()
 
 # and activate that DaemonPlugin
