@@ -2,15 +2,7 @@
 # -----------------------------------------------------------------------
 # Use PyLCD to display menus and players
 # -----------------------------------------------------------------------
-# $Id: lcd.py 10710 2008-05-09 17:43:31Z duncan $
-#
-# Notes:
-#    To activate, put the following line in local_conf.py:
-#       plugin.activate('lcd')
-# Todo:
-#    1) Use Threads. PyLCD is too blocking!
-#    2) Have Movie Player, TV Player and Image viewer to use LCD
-#    3) Better (and more) LCD screens.
+# $Id: lcd.py 11248 2009-01-14 18:23:22Z duncan $
 # -----------------------------------------------------------------------
 # Freevo - A Home Theater PC framework
 # Copyright (C) 2002 Krister Lagerstrom, et al.
@@ -32,6 +24,18 @@
 #
 # -----------------------------------------------------------------------
 
+"""
+Plug-in to display menus and players on a LCD display
+
+To activate, put the following line in local_conf.py::
+
+    plugin.activate('lcd')
+
+Todo:
+    1. Use Threads. PyLCD is too blocking!
+    2. Have Movie Player, TV Player and Image viewer to use LCD
+    3. Better (and more) LCD screens.
+"""
 
 from menu import MenuItem
 import copy
@@ -528,8 +532,8 @@ class PluginInterface(plugin.DaemonPlugin):
     """
     Display context info in LCD using lcdproc daemon.
 
-    @requires: lcdproc installed and LCDd running. U{http://lcdproc.sourceforge.net/}
-    @requires: pylcd installed U{http://www.schwarzvogel.de/software-pylcd.shtml}
+    Requires: lcdproc installed and LCDd running. U{http://lcdproc.sourceforge.net/}
+    Requires: pylcd installed U{http://www.schwarzvogel.de/software-pylcd.shtml}
 
     Also, your LCD dimensions must be supported. Right now it support: 4x20,
     4x40, 2x16, 2x20, 2x40. If you have one with different dimension, it will
@@ -552,7 +556,7 @@ class PluginInterface(plugin.DaemonPlugin):
     __author_email__     = 'barbieri@gmail.com'
     __maintainer__       = __author__
     __maintainer_email__ = __author_email__
-    __version__          = '$Revision: 10710 $'
+    __version__          = '$Revision: 11248 $'
 
     def __init__(self, height=None, width=None):
         """
@@ -564,8 +568,8 @@ class PluginInterface(plugin.DaemonPlugin):
             self.lcd = pylcd.client()
             cm = self.lcd.connect()
         except:
-            log.error(String(_('LCD plugin will not load! Maybe you don"t have LCDd (lcdproc daemon) running?')))
             self.disable = 1
+            self.reason = 'LCD plugin will not load! Maybe you don"t have LCDd (lcdproc daemon) running?'
             return
 
         if config.DEBUG > 0:
@@ -599,7 +603,8 @@ class PluginInterface(plugin.DaemonPlugin):
             self.animation_audioplayer_chars = ['-', '\\', '|', '/']
         else:
             self.disable = 1
-            log.warning('Unsupported LCDd version: %s' % (self.version,))
+            self.reason = 'Unsupported LCDd version: %s' % (self.version,)
+            return
 
         plugin.register(self, 'lcd')
 
@@ -613,10 +618,8 @@ class PluginInterface(plugin.DaemonPlugin):
             except UnicodeError:
                 self.lcd.widget_set('welcome', w, param)
 
-        self.lcd.screen_set('welcome', '-priority %s -duration 2 -heartbeat off' % \
-                             (self.prio_map['low']))
+        self.lcd.screen_set('welcome', '-priority %s -duration 2 -heartbeat off' % (self.prio_map['low']))
         self.last_screen = 'welcome'
-
         self.lsv = { } # will hold last screen value (lsv)
 
 
@@ -638,9 +641,10 @@ class PluginInterface(plugin.DaemonPlugin):
 
         # Check if audio is detached
         # When in detached mode, do not draw the player screen
-        if plugin.getbyname('audio.detachbar'):
-            if type == 'player' and plugin.getbyname('audio.detachbar').status != 0:
-                return
+        if type == 'player':
+            if plugin.getbyname('audio.detachbar'):
+                if plugin.getbyname('audio.detachbar').state != 1: #BAR_HIDE
+                    return
 
         if type == 'player':
             sname = '%s_%s' % (object.type, type)
@@ -660,31 +664,35 @@ class PluginInterface(plugin.DaemonPlugin):
             self.lsv = { } # reset last changed values
 
         if type == 'menu':
-            menu  = object.menustack[-1]
-            title = menu.selected.name
-            if isinstance(menu.selected, MenuItem):
-                title = _(title)
-            typeinfo = menu.selected.type
-            info = ''
+            try:
+                menu  = object.menustack[-1]
+                title = menu.selected.name
+                if isinstance(menu.selected, MenuItem):
+                    title = _(title)
 
-            if menu.selected.getattr('type'):
-                typeinfo = menu.selected.getattr('type')
+                typeinfo = menu.selected.type
+                info = ''
 
-            # get info:
-            if menu.selected.type and menu_info.has_key(menu.selected.type):
-                info = get_info(menu.selected, menu_info[menu.selected.type])
-                if menu_strinfo.has_key(menu.selected.type) and menu_strinfo[menu.selected.type]:
-                    if info:
-                        info += sep_str
-                    info += time.strftime(menu_strinfo[menu.selected.type])
+                if menu.selected.getattr('type'):
+                    typeinfo = menu.selected.getattr('type')
 
-            # specific things related with item type
-            if menu.selected.type == 'audio':
-                title = String(menu.selected.getattr('title'))
-                if not title:
-                    title = String(menu.selected.getattr('name'))
-                if menu.selected.getattr('trackno'):
-                    title = '%s - %s' % (String(menu.selected.getattr('trackno')), title)
+                # get info:
+                if menu.selected.type and menu_info.has_key(menu.selected.type):
+                    info = get_info(menu.selected, menu_info[menu.selected.type])
+                    if menu_strinfo.has_key(menu.selected.type) and menu_strinfo[menu.selected.type]:
+                        if info:
+                            info += sep_str
+                        info += time.strftime(menu_strinfo[menu.selected.type])
+
+                # specific things related with item type
+                if menu.selected.type == 'audio':
+                    title = String(menu.selected.getattr('title'))
+                    if not title:
+                        title = String(menu.selected.getattr('name'))
+                    if menu.selected.getattr('trackno'):
+                        title = '%s - %s' % (String(menu.selected.getattr('trackno')), title)
+            except:
+                title = ''
 
         elif type == 'player':
             player = object
