@@ -2,9 +2,12 @@
 # -----------------------------------------------------------------------
 # A plugin to obtain detailed weather forecast information
 # -----------------------------------------------------------------------
-# $Id: oneclick.py 10608 2008-04-05 12:01:56Z duncan $
+# $Id: oneclick.py 11520 2009-05-16 11:46:10Z duncan $
 #
 # Notes:
+#   There are two ways to get the radar map:
+#     - parse the link http://www.weather.com/outlook/travel/businesstraveler/map/<zip>?from=LAPmaps&bypassredirect=true
+#     - look in the http://cgi.weather.com/looper/archive/ directory for the latest image
 #
 # Todo:
 #   - i18n support
@@ -137,9 +140,9 @@ def wget(url):
     _debug_('wget(%s)' % (url), 2)
     txdata = None
     txheaders = {
-        'User-Agent' : 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1.13) Gecko/20080311 Firefox/2.0.0.13'
+        'User-Agent' : 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.10) Gecko/2009042316 Firefox/3.0.10'
     }
-    print 'getting url %r...' % (url)
+    _debug_('getting url %r...' % (url), DINFO)
     req = urllib2.Request(url, txdata, txheaders)
     try:
         t1 = time.time()
@@ -152,16 +155,16 @@ def wget(url):
         if response.msg == 'OK':
             return data
         _debug_('Downloaded "%s" in %.1f seconds' % (url, t2 - t1))
-    except urllib2.HTTPError, error:
-        print 'getting %r failed: %s' % (url, error)
-    except ValueError, error:
+    except urllib2.HTTPError, why:
+        _debug_('Download of %r failed: %s' % (url, why), DWARNING)
+    except ValueError, why:
         try:
             fd = open(url)
             data = fd.read()
             fd.close()
             return data
         except:
-            print 'invalid url %r failed: %s' % (url, error)
+            _debug_('invalid url %r failed: %s' % (url, why), DWARNING)
     return None
 
 
@@ -217,8 +220,8 @@ class PluginInterface(plugin.MainMenuPlugin):
         """
         """
         _debug_('PluginInterface.__init__()', 2)
-        if not config.USE_NETWORK:
-            self.reason = 'USE_NETWORK not enabled'
+        if not config.SYS_USE_NETWORK:
+            self.reason = 'SYS_USE_NETWORK not enabled'
             return
         if not config.ONECLICK_LOCATIONS:
             self.reason = 'ONECLICK_LOCATIONS not defined'
@@ -234,7 +237,7 @@ class PluginInterface(plugin.MainMenuPlugin):
             ('ONECLICK_URL_CURC', 'http://ff.1click.weather.com/weather/local/%s?cc=*%s', 'Current Conditions URL'),
             ('ONECLICK_URL_DAYF', 'http://ff.1click.weather.com/weather/local/%s?dayf=5%s', 'Day Forecast URL'),
             ('ONECLICK_URL_ELOC', 'http://ff.1click.weather.com/weather/local/%s?eloc=st', 'Extended Location URL'),
-            ('ONECLICK_URL_MAP',  'http://www.weather.com/weather/map/%s?from=LAPmaps', 'Radar Map URL')
+            ('ONECLICK_URL_MAP',  'http://www.weather.com/weather/map/%s?from=LAPmaps&bypassredirect=true', 'Radar Map URL')
         ]
 
     def items(self, parent):
@@ -280,7 +283,7 @@ class WeatherItem(Item):
         self.temperature = None
         self.feeling = None
         self.current_conditions = None
-        self.icon = None
+        self.day_icon = None
         self.pressure = None
         self.pressure_change = None
         self.wind_speed = None
@@ -336,8 +339,7 @@ class WeatherItem(Item):
         self.cacheCurrent = '%s/current.pickle' % (self.cacheDir)
         self.cacheForecast = '%s/forecast.pickle' % (self.cacheDir)
         self.mapFile = '%s/map.jpeg' % (self.cacheDir)
-        self.mapPage1 = '%s/mappage1.html' % (self.cacheDir)
-        self.mapPage2 = '%s/mappage2.html' % (self.cacheDir)
+        self.mapPage = '%s/mappage.html' % (self.cacheDir)
         if not os.path.isdir(self.cacheDir):
             os.mkdir(self.cacheDir, stat.S_IMODE(os.stat(config.FREEVO_CACHEDIR)[stat.ST_MODE]))
         self.last_update = 0
@@ -358,7 +360,7 @@ class WeatherItem(Item):
                 self.loadFromCache()
         except IOError, e:
             self.error = 1
-            print "failed to update data for '%s': %s" % (self.location, e)
+            _debug_("failed to update data for '%s': %s" % (self.location, e), DWARNING)
         else:
             # set the last update timestamp
             self.last_update = os.path.getmtime(self.cacheCurrent)
@@ -366,8 +368,8 @@ class WeatherItem(Item):
             # now convert the self.weatherData structure to parsable information
             try:
                 self.convertWeatherData()
-            except Exception, error:
-                print 'Failed to convert data for %s: %s' % (self.location, error)
+            except Exception, why:
+                _debug_('Failed to convert data for %s: %s' % (self.location, why), DWARNING)
 
 
     def needsRefresh(self):
@@ -392,16 +394,16 @@ class WeatherItem(Item):
             try:
                 elocationData = wget(self.url_eloc)
                 self.elocationData = elocationData
-            except Exception, error:
-                print 'Failed to get extended location data for %s: %s' % (self.location, error)
+            except Exception, why:
+                _debug_('Failed to get extended location data for %s: %s' % (self.location, why), DWARNING)
         else:
             self.elocationData = util.read_pickle(self.cacheElocation)
 
         try:
             self.currentData = wget(self.url_curc)
             #print 'currentData:', self.currentData
-        except Exception, error:
-            print 'Failed to get the current conditions data for %s: %s' % (self.location, error)
+        except Exception, why:
+            _debug_('Failed to get the current conditions data for %s: %s' % (self.location, why), DWARNING)
             if os.path.isfile(self.cacheCurrent):
                 self.currentData = util.read_pickle(self.cacheCurrent)
             else:
@@ -409,8 +411,8 @@ class WeatherItem(Item):
         try:
             self.forecastData = wget(self.url_dayf)
             #print 'forecastData:', self.forecastData
-        except Exception, error:
-            print 'Failed to get the forecast data for %s: %s' % (self.location, error)
+        except Exception, why:
+            _debug_('Failed to get the forecast data for %s: %s' % (self.location, why), DWARNING)
             if os.path.isfile(self.cacheForecast):
                 self.forecastData = util.read_pickle(self.cacheForecast)
             else:
@@ -439,8 +441,9 @@ class WeatherItem(Item):
                 try:
                     self.weatherMapData = wget(self.mapuri)
                     self.saveMapToCache()
-                except Exception, error:
-                    print 'Cannot download the map for "%s" from %s: %s' % (self.location, self.mapuri, error)
+                except Exception, why:
+                    _debug_('Cannot download the map for "%s" from %s: %s' % (self.location, self.mapuri, why),
+                        DWARNING)
                 return
             finally:
                 if GUI:
@@ -452,32 +455,20 @@ class WeatherItem(Item):
                 popup.show()
             # get the first web page
             weatherPage = wget(self.mapurl)
-            if config.DEBUG:
-                f = open(self.mapPage1, 'w')
+            if config.DEBUG and weatherPage:
+                f = open(self.mapPage, 'w')
                 f.write(weatherPage)
                 f.close()
             try:
                 # find link to map page
-                regexp = re.compile ('if \(isMinNS4\) var mapNURL = "([^"]*)";', re.IGNORECASE)
+                regexp = re.compile('NAME="mapImg"\s*SRC="([^"]*)"', re.IGNORECASE|re.MULTILINE)
                 results = regexp.search(weatherPage)
-                print 'weatherPage=%r' % (results.groups())
-                weatherPage2 = "http://www.weather.com/%s" % (results.groups()[0])
-
-                mapPage = wget(weatherPage2)
-                if config.DEBUG:
-                    f = open(self.mapPage2, 'w')
-                    f.write(weatherPage)
-                    f.close()
-                # find a link to the real weather map
-                regexp = re.compile('<img NAME="mapImg" SRC="(http://image.weather.com[^"]*jpg)"', re.IGNORECASE)
-                results = regexp.search(mapPage)
-                print 'mapPage=%r' % (results.groups())
                 self.mapuri = results.groups()[0]
                 self.weatherMapData = wget(self.mapuri)
                 self.saveMapToCache()
                 return
-            except Exception, error:
-                print 'Cannot download the map for "%s" from %s: %s' % (self.location, self.mapurl, error)
+            except Exception, why:
+                _debug_('Cannot download the map for "%s" from %s: %s' % (self.location, self.mapurl, why), DWARNING)
 
         finally:
             if GUI:
@@ -496,8 +487,8 @@ class WeatherItem(Item):
                 imgfd = os.open(self.mapFile, os.O_CREAT|os.W_OK)
                 os.write(imgfd, self.weatherMapData)
                 os.close(imgfd)
-        except Exception, error:
-            print 'failed saving weather map to cache "%s": %s' % (self.mapFile, error)
+        except Exception, why:
+            _debug_('failed saving weather map to cache "%s": %s' % (self.mapFile, why), DWARNING)
 
     def loadFromCache(self):
         """ load the data and the map from the cache """
@@ -508,8 +499,8 @@ class WeatherItem(Item):
 
         try:
             size = int(os.stat(self.mapFile)[6])
-        except Exception, error:
-            _debug_('failed loading weather map for "%s" from cache: %s' % (self.location, error), DWARNING)
+        except Exception, why:
+            _debug_('failed loading weather map for "%s" from cache: %s' % (self.location, why), DWARNING)
             pass
         else:
             imgfd = os.open(self.mapFile, os.R_OK)
@@ -579,7 +570,7 @@ class WeatherItem(Item):
     def getIcon(self):
         """ gets the current conditions icon """
         _debug_('getIcon()', 2)
-        return "%s" % (self.icon)
+        return "%s" % (self.day_icon)
 
     def getPressure(self):
         _debug_('getPressure()', 2)
@@ -593,7 +584,7 @@ class WeatherItem(Item):
 
     def getPressureChange(self):
         _debug_('getPressureChange()', 2)
-        return "%s" % (self.pressure_change)
+        return "%s" % (_(self.pressure_change))
 
     def getWind(self):
         _debug_('getWind()', 2)
@@ -601,7 +592,7 @@ class WeatherItem(Item):
             return "%s" % (self.wind_speed)
         elif self.wind_direction == 'VAR':
             return "%s %s %s %s" % (_('Variable'), _('at a rate of'), self.wind_speed, self.unit_s)
-        return "%s %s %s %s" % (self.wind_direction, _('at a rate of'), self.wind_speed, self.unit_s)
+        return "%s %s %s %s" % (_(self.wind_direction), _('at a rate of'), self.wind_speed, self.unit_s)
 
     def getWindDir(self):
         _debug_('getWindDir()', 2)
@@ -621,11 +612,11 @@ class WeatherItem(Item):
 
     def getUvIndex(self):
         _debug_('getUvIndex()', 2)
-        return "%s" % (self.uv_index)
+        return "%s" % (_(self.uv_index))
 
     def getUvType(self):
         _debug_('getUvType()', 2)
-        return "%s" % (self.uv_type)
+        return "%s" % (_(self.uv_type))
 
     def getDewPoint(self):
         _debug_('getDewPoint()', 2)
@@ -637,7 +628,7 @@ class WeatherItem(Item):
 
     def getMoonPhase(self):
         _debug_('getMoonPhase()', 2)
-        return "%s" % (self.moon_phase)
+        return "%s" % (_(self.moon_phase))
 
     def getSunrise(self):
         _debug_('getSunrise()', 2)
@@ -673,7 +664,7 @@ class WeatherItem(Item):
             self.city = dnam[0]
             self.state = ''
             self.country = dnam[1]
-        print 'city=%s, state=%s, country=%s' % (self.city, self.state, self.country)
+        _debug_('city=%s, state=%s, country=%s' % (self.city, self.state, self.country), DINFO)
 
         # reset variables
         self.date = []
@@ -701,8 +692,8 @@ class WeatherItem(Item):
         self.observation_station = current.cc.obst
         self.temperature = current.cc.tmp
         self.feeling = current.cc.flik
-        self.current_conditions = current.cc.t
-        self.icon = current.cc.icon
+        self.current_conditions = _(current.cc.t)
+        self.day_icon = current.cc.icon
         self.pressure = current.cc.bar.r
         self.pressure_change = current.cc.bar.d
         self.wind_speed = current.cc.wind.s
@@ -716,7 +707,7 @@ class WeatherItem(Item):
         self.moon_phase = current.cc.moon.t
 
         self.description = '%s %s %s' % (self.current_conditions, _("at"), self.getTemperature())
-        self.image = self.getDayImage(self.icon)
+        self.image = self.getDayImage(self.day_icon)
 
         # skip today in the days
         for day in forecast.dayf.days[1:]:
@@ -745,7 +736,7 @@ class WeatherItem(Item):
                 icon = os.path.join(WEATHER_DIR, WEATHER_ICONS[num][2])
         if not os.path.isfile(icon):
             icon = os.path.join(WEATHER_DIR, WEATHER_ICONS['na'][0])
-        print '%s: %s %s' % (num, icon, os.path.split(WEATHER_ICONS[num][0])[1])
+        _debug_('%s: %s %s' % (num, icon, os.path.split(WEATHER_ICONS[num][0])[1]))
         return icon
 
 
@@ -754,7 +745,7 @@ class WeatherItem(Item):
         _debug_('getMoonImage()', 2)
 
         icon = os.path.join(WEATHER_DIR, 'moons', '%s.png' % (num))
-        print '%s: %s' % (num, icon)
+        _debug_('%s: %s' % (num, icon), DINFO)
         return icon
 
 
@@ -835,7 +826,7 @@ class WeatherDetailHandler:
         self.subtitle = self.getSubtitle(self.skin_num)
 
         # Fire up splashscreen and load the plugins
-        skin.draw('oneclick', self)
+        skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
 
     def prevSkin(self):
         """decrements the skin number round to the last skin"""
@@ -873,21 +864,22 @@ class WeatherDetailHandler:
         elif event == 'MENU_SELECT':
             self.weather.getForecast(force=1)
             skin.clear()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         elif event in ('MENU_DOWN', 'MENU_RIGHT'):
             self.nextSkin()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         elif event in ('MENU_UP', 'MENU_LEFT'):
             self.prevSkin()
-            skin.draw('oneclick', self)
+            skin.draw('oneclick', self, blend=config.FREEVO_USE_ALPHABLENDING)
             return True
 
         return False
 
+
 if __name__ == '__main__':
     for location in config.ONECLICK_LOCATIONS:
         print location
@@ -896,6 +888,7 @@ if __name__ == '__main__':
     import sys
     sys.exit(1)
 
+
 class WeatherBaseScreen(skin.Area):
     """ A base class for weather screens to inherit from, provides common members+methods """
     def __init__(self):
@@ -915,8 +908,7 @@ class WeatherBaseScreen(skin.Area):
         self.xscale = lambda x: int(x * self.xmult)
         self.yscale = lambda y: int(y * self.ymult)
 
-        self.update_functions = (self.update_day, self.update_forecast,
-                                 self.update_week, self.update_map)
+        self.update_functions = (self.update_day, self.update_forecast, self.update_week, self.update_map)
 
     def day_item(self, x1, text, x2, value, y, align='left'):
         """ This is helper function for update_day
@@ -948,7 +940,6 @@ class WeatherBaseScreen(skin.Area):
         y += y_inc
         self.day_item(x_col1, _('UV Index'), x_col2, self.parent.weather.getUvType(), y)
 
-
         # draw current condition image
         x_start = self.content.x + self.xscale(480)
         y_start = self.content.y + self.yscale(40)
@@ -979,25 +970,25 @@ class WeatherBaseScreen(skin.Area):
 
         lines = []
         try:
-            lines.append('%s %s' % (_('As of:'), weather.getLastUpdated()))
-            lines.append('%s %s' % (_('at'), weather.getObservationStation()))
+            lines.append('%s: %s' % (_('As of'), weather.getLastUpdated()))
+            lines.append('%s: %s' % (_('at'), weather.getObservationStation()))
             lines.append('  %s' % (weather.getCurrentCondition()))
-            lines.append('  %s %s' % (_('Temperature:'), weather.getTemperature()))
-            lines.append('  %s %s' % (_('Dew Point:'), weather.getDewPoint()))
-            lines.append('  %s %s' % (_('Humidity:'), weather.getHumidity()))
-            lines.append('  %s %s' % (_('Visibility:'), weather.getVisibility()))
-            lines.append('  %s %s' % (_('Pressure:'), weather.getPressure()))
-            lines.append('  %s %s' % (_('Winds:'), weather.getWind()))
-            lines.append('%s' % (_('Tonight:')))
-            lines.append('  %s %s' % (_('Sunset:'), weather.getSunset()))
-            lines.append('  %s %s' % (_('Moon Phase:'), weather.getMoonPhase()))
-        except Exception, error:
-            print error
+            lines.append('  %s: %s' % (_('Temperature'), weather.getTemperature()))
+            lines.append('  %s: %s' % (_('Dew Point'), weather.getDewPoint()))
+            lines.append('  %s: %s' % (_('Humidity'), weather.getHumidity()))
+            lines.append('  %s: %s' % (_('Visibility'), weather.getVisibility()))
+            lines.append('  %s: %s' % (_('Pressure'), weather.getPressure()))
+            lines.append('  %s: %s' % (_('Winds'), weather.getWind()))
+            lines.append('%s:' % (_('Tonight')))
+            lines.append('  %s: %s' % (_('Sunset'), weather.getSunset()))
+            lines.append('  %s: %s' % (_('Moon Phase'), weather.getMoonPhase()))
+        except Exception, why:
+            _debug_(why, DWARNING)
             import traceback, sys
             output = apply(traceback.format_exception, sys.exc_info())
             output = ''.join(output)
             output = urllib.unquote(output)
-            print output
+            _debug(output)
 
         y = y_start
         for line in lines:
@@ -1010,8 +1001,8 @@ class WeatherBaseScreen(skin.Area):
             y_start = self.content.y + self.yscale(300)
             #self.draw_image(weather.getMoonImage(weather.getMoonIcon()),
             #(x_start, y_start, self.xscale(90), self.yscale(90)))
-        except Exception, error:
-            print error
+        except Exception, why:
+            _debug_(why, DWARNING)
 
 
     def week_item(self, x, y, text, font, width=90, align='center'):
@@ -1020,7 +1011,7 @@ class WeatherBaseScreen(skin.Area):
 
     def update_week(self):
         """ update the weeks forecast
-        @todo: this can be improved
+        TODO this can be improved
         """
         _debug_('update_week()', 2)
 
@@ -1033,13 +1024,13 @@ class WeatherBaseScreen(skin.Area):
             x = x_start + self.xscale(pos)
             y = y_start
 
-            self.week_item(x, y, Unicode(self.parent.weather.date[day]), self.key_font, 160)
+            self.week_item(x, y, _(Unicode(self.parent.weather.date[day])), self.key_font, 160)
 
             self.draw_image(self.parent.weather.weatherIcon[day],
                 (x, y + self.yscale(50), self.xscale(160), self.yscale(120)))
 
             y = y_start + self.yscale(200)
-            self.week_item(x, y, Unicode(self.parent.weather.weatherType[day]), self.small_font, 160)
+            self.week_item(x, y, _(Unicode(self.parent.weather.weatherType[day])), self.small_font, 160)
 
             y = y_start + self.yscale(240)
             self.week_item(x, y, _("LO"), self.val_font)
@@ -1083,3 +1074,213 @@ class WeatherBaseScreen(skin.Area):
 
 # create one instance of the WeatherType class
 skin.register ('oneclick', ('screen', 'subtitle', 'title', 'plugin', WeatherBaseScreen()))
+
+TRANLATIONS = [
+    _('Mostly Cloudy'),
+    _('Partly Cloudy'),
+    _('Light Rain'),
+    _('Showers'),
+    _('Rain'),
+    _('Fog'),
+    _('Few Showers'),
+    _('Mostly Sunny'),
+    _('Sunny'),
+    _('Scattered Flurries'),
+    _('Isolated T-Storms'),
+    _('Scattered Thunderstorms'),
+    _('Snow Showers'),
+    _('Few Snow Showers'),
+    _('Flurries'),
+    _('Thunder'),
+    _('Heavy Snow'),
+    _('Drizzle'),
+    _('Sct Flurries'),
+    _('Sct Strong Storms'),
+    _('Thunderstorms'),
+    _('Iso T-Storms'),
+    _('Sct T-Storms'),
+    _('Sct Snow Showers'),
+    _('to'),
+    _('Ice'),
+    _('Light Snow'),
+    _('Wintry Mix'),
+    _('Hvy Rain'),
+    _('Freezing Rain'),
+    _('T-Storms'),
+    _('Heavy Rain'),
+    _('Light Wintry Mix'),
+    _('Snow Shower'),
+    _('Strong Storms'),
+    _('Light Rain Shower'),
+    _('Light Rain with Thunder'),
+    _('Light Drizzle'),
+    _('Mist'),
+    _('Smoke'),
+    _('Haze'),
+    _('Light Snow Shower'),
+    _('Clear'),
+    _('A Few Clouds'),
+    _('Fair'),
+    _('Last updated: %s'),
+    _('Unlimited'),
+    _('%s (day: %s)'),
+    _('%s (type: %s)'),
+    _('Today, a high of %s degrees'),
+    _('and a low of %s degrees.'),
+    _('Currently, there is a humidity of'),
+    _('the winds are calm'),
+    _('Visibility will be unlimited today'),
+    _('There will be a visibility of'),
+    _('Error encountered while trying to download Radar map'),
+    _('Display program'),
+    _('Scheduling Failed'),
+    _('getFavorites failed: %s'),
+    _('Display favorite'),
+    _('Modify duplicate flag'),
+    _('Modify episodes flag'),
+    _('Modify Duplicate Flag'),
+    _('Save Failed, favorite was lost'),
+    _('Save Failed'),
+    _('Currently recording'),
+    _('Next recordings'),
+    _('Recordserver not found'),
+    _('Clear / Wind'),
+    _('Clouds Early / Clearing Late'),
+    _('Cloudy'),
+    _('Cloudy / Wind'),
+    _('Cloudy and Windy'),
+    _('Drifting Snow'),
+    _('Drizzle / Fog'),
+    _('Drizzle Late'),
+    _('E'),
+    _('ENE'),
+    _('ENE_short'),
+    _('ESE'),
+    _('ESE_short'),
+    _('E_short'),
+    _('Extreme'),
+    _('Few Showers / Wind'),
+    _('Few Snow'),
+    _('Few Snow Showers / Wind'),
+    _('First Quarter'),
+    _('Fog Early / Clouds Late'),
+    _('Fog Late'),
+    _('Foggy'),
+    _('Freezing Drizzle'),
+    _('Full'),
+    _('Heavy Rain / Wind'),
+    _('High'),
+    _('Isolated T-Storms / Wind'),
+    _('Last Quarter'),
+    _('Lawn and Garden Weather'),
+    _('Light Freezing Rain'),
+    _('Light Rain / Fog'),
+    _('Light Rain / Wind'),
+    _('Light Rain / Wind Late'),
+    _('Light Rain Early'),
+    _('Light Rain Late'),
+    _('Light Rain and Freezing Rain'),
+    _('Light Rain and Shower'),
+    _('Light Rain and Windy'),
+    _('Light Snow / Fog'),
+    _('Light Snow / Wind'),
+    _('Light Snow Early'),
+    _('Light Snow Late'),
+    _('Low'),
+    _('Mostly Clear'),
+    _('Mostly Cloudy / Wind'),
+    _('Mostly Cloudy and Windy'),
+    _('N'),
+    _('N/A'),
+    _('NE'),
+    _('NE_short'),
+    _('NNE'),
+    _('NNE_short'),
+    _('NNW'),
+    _('NNW_short'),
+    _('NW'),
+    _('NW_short'),
+    _('N_short'),
+    _('New'),
+    _('Overcast'),
+    _('PM Fog'),
+    _('PM Light Rain'),
+    _('PM Light Rain / Wind'),
+    _('PM Light Snow'),
+    _('PM Rain'),
+    _('PM Rain / Snow'),
+    _('PM Rain / Wind'),
+    _('PM Showers'),
+    _('PM Showers / Wind'),
+    _('PM Snow Showers'),
+    _('PM T-Storms'),
+    _('Partly Cloudy / Wind'),
+    _('Partly Cloudy and Windy'),
+    _('Rain / Freezing Rain'),
+    _('Rain / Snow'),
+    _('Rain / Snow / Wind'),
+    _('Rain / Snow Early'),
+    _('Rain / Snow Late'),
+    _('Rain / Snow Showers'),
+    _('Rain / Snow Showers / Wind'),
+    _('Rain / Snow Showers Late'),
+    _('Rain / Thunder'),
+    _('Rain / Wind'),
+    _('Rain / Wind Early'),
+    _('Rain Early'),
+    _('Rain Late'),
+    _('Rain Shower'),
+    _('Rain Shower and Windy'),
+    _('Rain and Snow'),
+    _('Rain to Wintry Mix'),
+    _('Rain to Wintry Mix / Wind'),
+    _('S'),
+    _('SE'),
+    _('SE_short'),
+    _('SSE'),
+    _('SSE_short'),
+    _('SSW'),
+    _('SSW_short'),
+    _('SW'),
+    _('SW_short'),
+    _('S_short'),
+    _('Scattered Showers'),
+    _('Scattered Snow'),
+    _('Scattered Snow Showers'),
+    _('Scattered T-Storms'),
+    _('Showers / Wind'),
+    _('Showers / Wind Early'),
+    _('Showers Early'),
+    _('Showers Late'),
+    _('Showers in the Vicinity'),
+    _('Snow'),
+    _('Snow / Wind'),
+    _('Snow Shower / Wind'),
+    _('Snow Showers / Wind Early'),
+    _('Snow Showers Early'),
+    _('Snow Showers Late'),
+    _('Snow and Freezing Rain'),
+    _('Snow to Rain'),
+    _('Snow to Wintry Mix'),
+    _('Snow to Wintry Mix / Wind'),
+    _('Storm'),
+    _('Sunny / Wind'),
+    _('T-Showers'),
+    _('T-Storms Early'),
+    _('Very High'),
+    _('W'),
+    _('WNW'),
+    _('WNW_short'),
+    _('WSW'),
+    _('WSW_short'),
+    _('W_short'),
+    _('Waning Crescent'),
+    _('Waning Gibbous'),
+    _('Waxing Crescent'),
+    _('Waxing Gibbous'),
+    _('Wintry Mix / Wind'),
+    _('Wintry Mix / Wind Early'),
+    _('Wintry Mix to Snow'),
+    _('rising'),
+    _('steady'),
+]
