@@ -2,9 +2,9 @@
 # -----------------------------------------------------------------------
 # class and helpers for fxd/imdb generation
 # -----------------------------------------------------------------------
-# $Id: fxdimdb.py 10716 2008-05-11 10:50:48Z duncan $
+# $Id: fxdimdb.py 11563 2009-05-25 15:39:56Z duncan $
 #
-# Notes: see http://pintje.servebeer.com/fxdimdb.html for documentatio,
+# Notes: see http://pintje.servebeer.com/fxdimdb.html for documentation,
 # Todo:
 # - add support making fxds without imdb (or documenting it)
 # - webradio support?
@@ -32,8 +32,8 @@
 
 
 # python has no data hiding, but this is the intended use...
-# subroutines completly in lowercase are regarded as more "private" functions
-# subRoutines are regarded as public
+# subroutines completely in lowercase are regarded as more "private" functions
+# sub-routines are regarded as public
 
 #some data
 __author__ = "den_RDC (rdc@kokosnoot.com)"
@@ -47,13 +47,14 @@ import urllib, urllib2, urlparse
 import sys
 import codecs
 import os
-from BeautifulSoup import BeautifulSoup
+import traceback
+from BeautifulSoup import BeautifulSoup, NavigableString
+import HTMLParser
 
 import config
 import util
 
 import kaa.metadata as mmpython
-#Constants
 
 try:
     import freevo.version as version
@@ -84,11 +85,11 @@ class FxdImdb:
         """Initialise class instance"""
 
         # these are considered as private variables - don't mess with them unless
-        # no other choise is given
-        # fyi, the other choice always exists : add a subroutine or ask :)
+        # no other choice is given
+        # FYI, the other choice always exists : add a subroutine or ask :)
 
-        self.imdb_id_list = []
-        self.imdb_id = None
+        self.id_list = []
+        self.id = None
         self.isdiscset = False
         self.title = ''
         self.info = {}
@@ -99,10 +100,6 @@ class FxdImdb:
 
         self.fxdfile = None # filename, full path, WITHOUT extension
 
-        self.season = None  # used if the file is a tv serie
-        self.episode = None # used if the file is a tv serie
-        self.newid = None   # used if the file is a tv serie
-
         self.append = False
         self.device = None
         self.regexp = None
@@ -124,18 +121,84 @@ class FxdImdb:
         self.image_url_handler['www.impawards.com'] = self.impawards
 
 
-    def searchImdb(self, name):
+    def guessImdb(self, filename, label=False):
+        """Guess possible imdb movies from file name. Same return as searchImdb"""
+        _debug_('guessImdb(filename=%r, label=%r)' % (filename, label))
+
+        # Special name rule for the encoding server
+        m = re.compile('DVD \[([^]]*).*')
+        res = m.search(filename)
+        if res:
+            name = res.group(1)
+        else:
+            name = filename
+
+        # is this a series with season and episode number?
+        # if so we will remember season and episode but will take it off from name
+
+        # find <season>X<episode>
+        season = ''
+        episode = ''
+        m = re.compile('([0-9]+)[xX]([0-9]+)')
+        res = m.search(name)
+        if res:
+            name = re.sub('%s.*' % res.group(0), '', name)
+            season = str(int(res.group(1)))
+            episode = str(int(res.group(2)))
+
+        # find S<season>E<episode>
+        m = re.compile('[sS]([0-9]+)[eE]([0-9]+)')
+        res = m.search(name)
+        if res:
+            name = re.sub('%s.*' % res.group(0), '', name)
+            season = str(int(res.group(1)))
+            episode = str(int(res.group(2)))
+
+        name = vfs.basename(vfs.splitext(name)[0])
+        name = re.sub('([a-z])([A-Z])', point_maker, name)
+        name = re.sub('([a-zA-Z])([0-9])', point_maker, name)
+        name = re.sub('([0-9])([a-zA-Z])', point_maker, name.lower())
+        name = re.sub(',', ' ', name)
+
+        _debug_('name=%s season=%s episode=%s' % (name, season, episode))
+        if label:
+            for r in config.IMDB_REMOVE_FROM_LABEL:
+                try:
+                    name = re.sub(r, '', name)
+                except Exception, e:
+                    print e
+        else:
+            for r in config.IMDB_REMOVE_FROM_NAME:
+                try:
+                    name = re.sub(r, '', name)
+                except Exception, e:
+                    print e
+
+        parts = re.split('[\._ -]', name)
+        name = ''
+        for p in parts:
+            if not p.lower() in config.IMDB_REMOVE_FROM_SEARCHSTRING and not re.search('[^0-9A-Za-z]', p):
+                # originally: not re.search(p, '[A-Za-z]'):
+                # not sure what's meant with that
+                name += '%s ' % p
+        return self.searchImdb(name, season, episode)
+
+
+    def searchImdb(self, name, season=None, episode=None):
         """
         Search IMDB for the name
 
         @param name: name to search for
         @type name: string
-        @returns: id list with tuples (id , name, year, type)
+        @returns: id list with tuples (id, name, year, type)
         """
+        _debug_('searchImdb(name=%r, season=%r, episode=%r)' % (name, season, episode))
+        tv_marker = (season and episode) and '"' or ''
+        name = tv_marker+name.strip().strip(tv_marker)+tv_marker
         _debug_('searching imdb for "%s"' % (name))
         url = 'http://us.imdb.com/Tsearch?title=%s&restrict=Movies+and+TV' % urllib.quote(str(name))
         url = 'http://www.imdb.com/find?s=tt;site=aka;q=%s' % urllib.quote(str(name))
-        _debug_('url="%s"' % (url))
+        _debug_('url="%s"' % (url,))
 
         req = urllib2.Request(url, txdata, txheaders)
         searchstring = name
@@ -144,21 +207,32 @@ class FxdImdb:
             response = urllib2.urlopen(req)
         except urllib2.HTTPError, error:
             raise FxdImdb_Net_Error("IMDB unreachable : " + error)
-            return None
 
-        _debug_('response.url="%s"' % (response.geturl()))
+        response_url = response.geturl()
+        _debug_('response.url="%s"' % (response_url,))
 
-        m=re.compile('/title/tt([0-9]*)/')
-        idm = m.search(response.geturl())
+        m = re.compile('/title/tt([0-9]*)/')
+        idm = m.search(response_url)
         if idm: # Direct Hit
+            id = idm.group(1)
+            if season and episode:
+                id = self.getIMDBid(id, season, episode)
             response.close()
-            return [(idm.group(1), name.title(), u'', '' )]
+            return [(id, name.title(), u'', '' )]
 
-        data = self.parsesearchdata(response)
+        if config.DEBUG >= 2:
+            tmpfilename = '/tmp/%s.html' % name.replace(' ', '_').lower()
+            tmp = open(tmpfilename, 'w')
+            tmp.write(response.read())
+            tmp.close()
+            response.close()
+            response = open(tmpfilename, 'r')
+
+        data = self.parsesearchdata(response, url=response_url)
         response.close()
 
-        _debug_('imdb_id_list has %s items' % (len(self.imdb_id_list)))
-        if len(self.imdb_id_list) > 20:
+        _debug_('id_list has %s items' % (len(self.id_list)))
+        if len(self.id_list) > 20:
             # too many results, check if there are stupid results in the list
             words = []
 
@@ -171,61 +245,205 @@ class FxdImdb:
 
             # at least one word has to be in the result
             new_list = []
-            for result in self.imdb_id_list:
+            for result in self.id_list:
                 appended = False
                 for search_word in words:
-                    if not appended and result[1] and \
-                           result[1].lower().find(search_word.lower()) != -1:
-                        new_list.append(result)
+                    if not appended and result[1] and result[1].lower().find(search_word.lower()) != -1:
+                        new_list.append(tuple([util.unescape(i) for i in result]))
                         appended = True
-            self.imdb_id_list = new_list
-            _debug_('imdb_id_list has now %s items' % (len(self.imdb_id_list)))
-        return self.imdb_id_list
+            self.id_list = new_list
+            _debug_('id_list has now %s items' % (len(self.id_list)))
+        return self.id_list
 
 
-    def setImdbId(self, id, season=None, episode=None):
+    def getIMDBid(self, id, season=None, episode=None):
         """id (number)
         Set an imdb_id number for object, and fetch data"""
 
-        self.imdb_id = id
+        self.id = id
 
-        self.season = season
-        self.episode = episode
-
-        if self.season and self.episode:
-            # This is a tv serie, lets use a special search
-            url = 'http://us.imdb.com/title/tt%s/episodes' % id
+        _debug_('id=%s season=%s episode=%s' % (id, season, episode))
+        if season is None or episode is None:
+            episodeid = id
+        else:
+            # This is a TV series, lets use a special search
+            url = 'http://www.imdb.com/title/tt%s/episodes' % id
+            _debug_('url=%r' % (url,))
             req = urllib2.Request(url, txdata, txheaders)
 
             try:
                 idpage = urllib2.urlopen(req)
-            except urllib2.HTTPError, error:
-                raise FxdImdb_Net_Error("IMDB unreachable" + error)
-                return None
+            except urllib2.HTTPError, why:
+                raise FxdImdb_Net_Error('IMDB unreachable' + str(why))
 
-            newid = self.findepisode(idpage)
-
-            if newid:
-                self.imdb_id = newid
-                self.newid = newid
+            episodeid = self.find_episode(idpage, id, season, episode)
             idpage.close()
 
         # do the standard search
-        url = 'http://us.imdb.com/Title?%s' % self.imdb_id
+        url = 'http://www.imdb.com/title/tt%s' % episodeid
+        _debug_('url=%r' % (url,))
         req = urllib2.Request(url, txdata, txheaders)
 
         try:
             idpage = urllib2.urlopen(req)
-        except urllib2.HTTPError, error:
-            raise FxdImdb_Net_Error("IMDB unreachable" + error)
-            return None
+        except urllib2.HTTPError, why:
+            raise FxdImdb_Net_Error('IMDB unreachable' + str(why))
 
-        self.parsedata(idpage, id)
+        if config.DEBUG >= 2:
+            tmpfilename = '/tmp/%s.html' % id
+            tmp = open(tmpfilename, 'w')
+            tmp.write(idpage.read())
+            tmp.close()
+            idpage.close()
+            idpage = open(tmpfilename, 'r')
 
+        self.parse_data(idpage, id, episodeid, season, episode)
         idpage.close()
 
+        _debug_('episodeid=%s' % (episodeid,))
+        return episodeid
 
-    def setFxdFile(self, fxdfilename = None, overwrite = False):
+
+    def parse_data(self, results, id, episodeid, season, episode):
+        """results (imdb html page), imdb_id
+        Returns tuple of (title, info(dict), image_urls)"""
+
+        dvd = 0
+
+        try:
+            soup = BeautifulSoup(results.read(), convertEntities='xml')
+        except UnicodeDecodeError:
+            print "Unicode error: check that /usr/lib/python2.x/site.py has the correct default encoding"
+            traceback.print_exc()
+            return (None, None, None)
+
+        # The parse tree can be now reduced by, everything outside this is not required:
+        main = soup.find('div', {'id': 'tn15main'})
+        #title = soup.title
+        title = soup.find('h1')
+        #this no longer works
+        #image = soup.find('img', { 'title':title.next.strip() })
+        #if image:
+        #    self.info['image'] = image['src']
+
+        self.title = title.next.strip()
+
+        #is this a series? series pages a little different
+        try:
+            self.title = "%s - S%sE%02d - %s" % (self.title, season, int(episode), title.find('em').string.strip())
+            self.info['title'] = self.title
+            y = title.find('em').next.next.string.strip()
+            self.info['year'] = y[1:-1]
+        except (AttributeError, TypeError, ValueError):
+            self.info['title'] = self.title
+            self.info['year'] = title.find('a').string.strip()
+
+        # Find the <div> with class info, each <h5> under this provides info
+        for info in main.findAll('div', {'class' : 'info'}):
+            infoh5 = info.find('h5')
+            if not infoh5:
+                continue
+            try:
+                infostr = infoh5.next
+                key = infostr.string.strip(':').lower().replace(' ', '_')
+                nextsibling = nextsibling = infoh5.nextSibling.strip()
+                sections = info.findAll('a', { 'href' : re.compile('/Sections') })
+                lists = info.findAll('a', { 'href' : re.compile('/List') })
+                if len(nextsibling) > 0:
+                    self.info[key] = nextsibling
+                elif len(sections) > 0:
+                    items = []
+                    for item in sections:
+                        items.append(item.string)
+                    self.info[key] = ' / '.join(items)
+                elif len(lists) > 0:
+                    items = []
+                    for item in lists:
+                        items.append(item.string)
+                    self.info[key] = ' / '.join(items)
+            except:
+                pass
+
+        # Find Plot Outline/Summary:
+        # Normally the tag is named "Plot Outline:" - however sometimes
+        # the tag is "Plot Summary:" or just "Plot:". Search for all strings.
+        imdb_result = soup.find(text='Plot Outline:')
+        if not imdb_result:
+            imdb_result = soup.find(text='Plot Summary:')
+        if not imdb_result:
+            imdb_result = soup.find(text='Plot:')
+        if imdb_result:
+            self.info['plot'] = imdb_result.next.strip()
+        else:
+            self.info['plot'] = u''
+
+        # Find tagline - sometimes the tagline is missing.
+        # Use an empty string if no tagline could be found.
+        imdb_result = soup.find(text='Tagline:')
+        if imdb_result:
+            self.info['tagline'] = imdb_result.next.strip()
+        else:
+            self.info['tagline'] = u''
+
+        rating = soup.find(text='User Rating:').findNext(text=re.compile('/10'))
+        if rating:
+            votes = rating.findNext('a')
+            self.info['rating'] = rating.strip() + ' (' + votes.string.strip() + ')'
+        else:
+            self.info['rating'] = ''
+
+        runtime = soup.find(text='Runtime:')
+        if runtime and runtime.next:
+            self.info['runtime'] = runtime.next.strip()
+        else:
+            self.info['runtime'] = ''
+
+        # Replace special characters in the items
+        for (k,v) in self.info.items():
+            self.info[k] = self.convert_entities(v)
+
+        if config.DEBUG >= 2:
+            for (k,v) in self.info.items():
+                _debug_('items=%s:%s' % (k, v))
+            _debug_('id="%s", dvd="%s"' % (id, dvd))
+            _debug_(self.info, 2)
+
+        # Add impawards.com poster URLs.
+        self.impawardsimages(self.info['title'], self.info['year'])
+
+        # Add images from IMDB database. These images are much smaller than
+        # the impawards ones.
+        if not id:
+            return (self.title, self.info, self.image_urls)
+
+        if not dvd:
+            url = 'http://us.imdb.com/title/tt%s/dvd' % id
+            _debug_('url="%s"' % (url))
+            req = urllib2.Request(url, txdata, txheaders)
+
+            try:
+                r = urllib2.urlopen(req)
+                soup.feed(r.read())
+                r.close()
+                divs = soup.findAll('table', { 'class' : 'dvd_section' })
+                for div in divs:
+                    image = div.find('img')
+                    if image['src'].find('http') < 0:
+                        continue
+                    self.image_urls += [ image['src'] ]
+            except urllib2.HTTPError, error:
+                pass
+            except UnicodeDecodeError:
+                # FIXME:
+                # This is a bad hack. Some character could not be converted to ascii.
+                # We ignore these errors as it does not really affect the FXD output.
+                pass
+        _debug_('image_urls=%s' % (self.image_urls))
+
+        return (self.title, self.info, self.image_urls)
+
+
+    def setFxdFile(self, fxdfilename=None, overwrite=False):
         """
         fxdfilename (string, full path)
         Set fxd file to write to, may be omitted, may be an existing file
@@ -239,8 +457,7 @@ class FxdImdb:
 
         else:
             if self.isdiscset == True:
-                self.fxdfile = vfs.join(config.OVERLAY_DIR, 'disc-set',
-                                        self.getmedia_id(self.device))
+                self.fxdfile = vfs.join(config.OVERLAY_DIR, 'disc-set', self.getmedia_id(self.device))
             else:
                 self.fxdfile = vfs.splitext(file)[0]
 
@@ -254,8 +471,7 @@ class FxdImdb:
             self.append = False
 
         # XXX: add this back in without using parseMovieFile
-        # if self.append == True and \
-        #    parseMovieFile(self.fxdfile + '.fxd', None, []) == []:
+        # if self.append == True and parseMovieFile(self.fxdfile + '.fxd', None, []) == []:
         #     raise FxdImdb_XML_Error("FXD file to be updated is invalid, please correct it.")
 
         if not vfs.isdir(vfs.dirname(self.fxdfile)):
@@ -366,67 +582,6 @@ class FxdImdb:
         return 0
 
 
-    def guessImdb(self, filename, label=False):
-        """Guess possible imdb movies from filename. Same return as searchImdb"""
-
-        # Special name rule for the encoding server
-        m = re.compile('DVD \[([^]]*).*')
-        res = m.search(filename)
-        if res:
-            name = res.group(1)
-        else:
-            name = filename
-
-
-
-        # is this a serie with season and episode number?
-        # if so we will remember season and episode but will take it off from name
-
-        # find SeasonXepisodeNumber
-        m = re.compile('([0-9]+)[xX]([0-9]+)')
-        res = m.search(name)
-        if res:
-            name = re.sub('%s.*' % res.group(0), '', name)
-            self.season = str(int(res.group(1)))
-            self.episode = str(int(res.group(2)))
-
-        # find S<season>E<episode>
-        m = re.compile('[sS]([0-9]+)[eE]([0-9]+)')
-        res = m.search(name)
-        if res:
-            name = re.sub('%s.*' % res.group(0), '', name)
-            self.season = str(int(res.group(1)))
-            self.episode = str(int(res.group(2)))
-
-        name  = vfs.basename(vfs.splitext(name)[0])
-        name  = re.sub('([a-z])([A-Z])', point_maker, name)
-        name  = re.sub('([a-zA-Z])([0-9])', point_maker, name)
-        name  = re.sub('([0-9])([a-zA-Z])', point_maker, name.lower())
-        name  = re.sub(',', ' ', name)
-
-        if label:
-            for r in config.IMDB_REMOVE_FROM_LABEL:
-                try:
-                    name = re.sub(r, '', name)
-                except Exception, e:
-                    print e
-        else:
-            for r in config.IMDB_REMOVE_FROM_NAME:
-                try:
-                    name = re.sub(r, '', name)
-                except Exception, e:
-                    print e
-
-        parts = re.split('[\._ -]', name)
-        name = ''
-        for p in parts:
-            if not p.lower() in config.IMDB_REMOVE_FROM_SEARCHSTRING and \
-                   not re.search('[^0-9A-Za-z]', p):
-                # originally: not re.search(p, '[A-Za-z]'):
-                # not sure what's meant with that
-                name += '%s ' % p
-        return self.searchImdb(name)
-
 
 #------ private functions below .....
 
@@ -450,25 +605,23 @@ class FxdImdb:
             raise FxdImdb_IO_Error("Writing FXD file failed : " + str(error))
             return
 
-        #header
         i.write("<?xml version=\"1.0\" ?>\n<freevo>\n")
         i.write("  <copyright>\n" +
                 "    The information in this file are from the Internet Movie Database (IMDb).\n" +
                 "    Please visit http://www.imdb.com for more information.\n")
-        i.write("    <source url=\"http://www.imdb.com/title/tt%s\"/>\n"  % self.imdb_id +
-                "  </copyright>\n")
-        #disc-set
+        if self.id:
+            i.write("    <source url=\"http://www.imdb.com/title/tt%s\"/>\n" % self.id)
+        i.write("  </copyright>\n")
+
         i.write("  <disc-set title=\"%s\">\n" % self.str2XML(self.title))
-        #disc
         i.write("    <disc")
         if self.device:
             i.write(" media-id=\"%s\"" % self.str2XML(self.getmedia_id(self.device)))
         elif self.regexp:
             i.write(" label-regexp=\"%s\"" % self.str2XML(self.regexp))
         if self.mpl_global_opt:
-            i.write(" mplayer-options=\"%s\">" % self.str2XML(self.mpl_global_opt))
-        else: i.write(">")
-        #file-opts
+            i.write(" mplayer-options=\"%s\"" % self.str2XML(self.mpl_global_opt))
+        i.write(">")
         if self.file_opts:
             i.write("\n")
             for opts in self.file_opts:
@@ -478,17 +631,14 @@ class FxdImdb:
             i.write("    </disc>\n")
         else: i.write("    </disc>\n")
 
-        #image
         if self.image:
             i.write("    <cover-img source=\"%s\">" % self.str2XML(self.image_url))
             i.write("%s</cover-img>\n" % self.str2XML(self.image))
-        #print info
         i.write(self.print_info())
-
-        #close tags
         i.write("  </disc-set>\n")
         i.write("</freevo>\n")
 
+        # now we need to rebuild the cache
         util.touch(os.path.join(config.FREEVO_CACHEDIR, 'freevo-rebuild-database'))
 
 
@@ -496,19 +646,19 @@ class FxdImdb:
         """Write <movie> to fxd file"""
 
         try:
-            i = vfs.codecs_open( (self.fxdfile + '.fxd') , 'w', encoding='utf-8')
+            i = vfs.codecs_open((self.fxdfile+'.fxd') , 'w', encoding='utf-8')
         except IOError, error:
             raise FxdImdb_IO_Error("Writing FXD file failed : " + str(error))
             return
 
         #header
         i.write("<?xml version=\"1.0\" ?>\n<freevo>\n")
-        i.write("  <copyright>\n" +
-                "    The information in this file are from the Internet " +
-                "Movie Database (IMDb).\n" +
-                "    Please visit http://www.imdb.com for more information.\n")
-        i.write("    <source url=\"http://www.imdb.com/title/tt%s\"/>\n"  % self.imdb_id +
-                "  </copyright>\n")
+        i.write("  <copyright>\n")
+        i.write("    The information in this file are from the Internet Movie Database (IMDb).\n")
+        i.write("    Please visit http://www.imdb.com for more information.\n")
+        if self.id:
+            i.write("    <source url=\"http://www.imdb.com/title/tt%s\"/>\n" % self.id)
+        i.write("  </copyright>\n")
         # write movie
         i.write("  <movie title=\"%s\">\n" % self.str2XML(self.title))
         #image
@@ -517,8 +667,7 @@ class FxdImdb:
             i.write("%s</cover-img>\n" % self.str2XML(self.image))
         #video
         if self.mpl_global_opt:
-            i.write("    <video mplayer-options=\"%s\">\n" % \
-                    self.str2XML(self.mpl_global_opt))
+            i.write("    <video mplayer-options=\"%s\">\n" % self.str2XML(self.mpl_global_opt))
         else: i.write("    <video>\n")
         # videos
         i.write(self.print_video())
@@ -538,16 +687,14 @@ class FxdImdb:
         util.touch(os.path.join(config.FREEVO_CACHEDIR, 'freevo-rebuild-database'))
 
 
-
     def update_movie(self):
-        """Updates an existing file, adds exftra dvd|vcd|file and variant tags"""
+        """Updates an existing file, adds extra dvd|vcd|file and variant tags"""
         passedvid = False
         #read existing file in memory
         try:
             file = vfs.open(self.fxdfile + '.fxd')
         except IOError, error:
             raise FxdImdb_IO_Error("Updating FXD file failed : " + str(error))
-            return
 
         content = file.read()
         file.close()
@@ -615,8 +762,7 @@ class FxdImdb:
             if regexp_discset_end.match(line):
                 file.write("    <disc")
                 if self.device:
-                    file.write(" media-id=\"%s\"" % \
-                               self.str2XML(self.getmedia_id(self.device)))
+                    file.write(" media-id=\"%s\"" % self.str2XML(self.getmedia_id(self.device)))
                 elif self.regexp:
                     file.write(" label-regexp=\"%s\"" % self.str2XML(self.regexp))
                 if self.mpl_global_opt:
@@ -627,8 +773,7 @@ class FxdImdb:
                     file.write("\n")
                     for opts in self.file_opts:
                         mplopts, fname = opts
-                        file.write("      <file-opt mplayer-options=\"%s\">" % \
-                                   self.str2XML(mplopts))
+                        file.write("      <file-opt mplayer-options=\"%s\">" % self.str2XML(mplopts))
                         file.write("%s</file-opt>\n" % self.str2XML(fname))
                     file.write("    </disc>\n")
                 else: file.write("    </disc>\n")
@@ -640,29 +785,42 @@ class FxdImdb:
         util.touch(os.path.join(config.FREEVO_CACHEDIR, 'freevo-rebuild-database'))
 
 
-    def parsesearchdata(self, results, id=0):
-        """results (imdb html page), imdb_id
-        Returns tuple of (title, info(dict), image_urls)"""
+    def parsesearchdata(self, results, url=None, id=0):
+        """
+        results (imdb html page), imdb_id
 
-        self.imdb_id_list = []
-        m=re.compile('/title/tt([0-9]*)/')
-        y=re.compile('\(([^)]+)\)')
-        soup = BeautifulSoup(results.read(), convertEntities='xml')
+        @returns: tuple of (title, info(dict), image_urls)
+        """
+        _debug_('parsesearchdata(results=%r, url=%r, id=%r)' % (results, url, id))
+
+        self.id_list = []
+        m = re.compile('/title/tt([0-9]*)/')
+        y = re.compile('\(([^)]+)\)')
+        try:
+            soup = BeautifulSoup(results.read(), convertEntities='xml')
+        except HTMLParser.HTMLParseError, why:
+            traceback.print_exc()
+            _debug_('Cannot parse %r: %s' % (url, why), DWARNING)
+            return self.id_list
+        except Exception, why:
+            traceback.print_exc()
+            _debug_('Cannot parse %r: %s' % (url, why), DWARNING)
+            return self.id_list
         items = soup.findAll('a', href=re.compile('/title/tt'))
         ids = set([])
         for item in items:
             idm = m.search(item['href'])
             if not idm:
                 continue
-            yrm = y.findall(item.next.next)
-            #print yrm
+            if isinstance(item.next.next, NavigableString):
+                yrm = y.findall(item.next.next)
 
             id = idm.group(1)
             name = item.string
             # skip empty names
             if not name:
                 continue
-            # skip diplicate ids
+            # skip duplicate ids
             if id in ids:
                 continue
             ids.add(id)
@@ -670,171 +828,34 @@ class FxdImdb:
             type = len(yrm) > 1 and yrm[1] or ''
             #print 'url', item['href']
             #print item.parent.findChildren(text=re.compile('[^ ]'))
-            self.imdb_id_list += [ ( id, name, year, type ) ]
+            self.id_list += [ ( id, name, year, type ) ]
 
-        for item in self.imdb_id_list:
+        for item in self.id_list:
             _debug_('%r' % (item,), 2)
-        return self.imdb_id_list
+        return self.id_list
 
-    def findepisode(self, results):
+
+    def find_episode(self, results, id, season, episode):
         """results (imdb html page)
-        Returns a new id for setImdbId with tv serie episode data"""
-
-        newid = None
-
+        Returns a new id for getIMDBid with TV series episode data
+        """
         try:
             soup = BeautifulSoup(results.read(), convertEntities='xml')
         except UnicodeDecodeError:
             print "Unicode error; check that /usr/lib/python2.x/site.py has the correct default encoding"
             pass
 
-        m = re.compile('.*Season %s, Episode %s.*\/tt([0-9]+)' % (self.season, self.episode))
+        m = re.compile('.*Season %s, Episode %s.*\/tt([0-9]+)' % (season, episode))
 
-        for episode in soup.findAll('h4'):
+        for episode in soup.findAll('h3'):
             info = m.search(str(episode))
-            if not info:
-                continue
-            newid = info.group(1)
-            break
+            if info:
+                episodeid = info.group(1)
+                _debug_('episodeid=%r' % (episodeid,))
+                return episodeid
 
-        return(newid)
-
-    def parsedata(self, results, id=0):
-        """results (imdb html page), imdb_id
-        Returns tuple of (title, info(dict), image_urls)"""
-
-        dvd = 0
-
-        try:
-            soup = BeautifulSoup(results.read(), convertEntities='xml')
-        except UnicodeDecodeError:
-            print "Unicode error; check that /usr/lib/python2.x/site.py has the correct default encoding"
-            return (None, None, None)
-
-        # The parse tree can be now reduced by, everything outside this is not required:
-        main = soup.find('div', {'id': 'tn15main'})
-        #title = soup.title
-        title = soup.find('h1')
-        #this no longer works
-        #image = soup.find('img', { 'title':title.next.strip() })
-        #if image:
-        #    self.info['image'] = image['src']
-
-        self.title = title.next.strip()
-
-        #is this a serie? series pages a little different
-        if self.newid:
-            self.title = self.title + " - %sx%.2d - %s" % (self.season, \
-                         int(self.episode), title.find('em').string.strip() )
-            self.info['title'] = self.title
-            y = title.find('em').next.next.string.strip()
-            self.info['year'] = y[1:-1]
-
-        else:
-            self.info['title'] = self.title
-            self.info['year'] = title.find('a').string.strip()
-
-        # Find the <div> with class info, each <h5> under this provides info
-        for info in main.findAll('div', {'class' : 'info'}):
-            infoh5 = info.find('h5')
-            if not infoh5:
-                continue
-            try:
-                infostr = infoh5.next
-                key = infostr.string.strip(':').lower().replace(' ', '_')
-                nextsibling = nextsibling = infoh5.nextSibling.strip()
-                sections = info.findAll('a', { 'href' : re.compile('/Sections') })
-                lists = info.findAll('a', { 'href' : re.compile('/List') })
-                if len(nextsibling) > 0:
-                    self.info[key] = nextsibling
-                elif len(sections) > 0:
-                    items = []
-                    for item in sections:
-                        items.append(item.string)
-                    self.info[key] = ' / '.join(items)
-                elif len(lists) > 0:
-                    items = []
-                    for item in lists:
-                        items.append(item.string)
-                    self.info[key] = ' / '.join(items)
-            except:
-                pass
-
-        # Find Plot Outline/Summary:
-        # Normally the tag is named "Plot Outline:" - however sometimes
-        # the tag is "Plot Summary:". Search for both strings.
-        imdb_result = soup.find(text='Plot Outline:')
-        if not imdb_result:
-            imdb_result = soup.find(text='Plot Summary:')
-        if imdb_result:
-            self.info['plot'] = imdb_result.next.strip()
-        else:
-            self.info['plot'] = u''
-
-        # Find tagline - sometimes the tagline is missing.
-        # Use an empty string if no tagline could be found.
-        imdb_result = soup.find(text='Tagline:')
-        if imdb_result:
-            self.info['tagline'] = imdb_result.next.strip()
-        else:
-            self.info['tagline'] = u''
-
-        rating = soup.find(text='User Rating:').findNext(text=re.compile('/10'))
-        if rating:
-            votes = rating.findNext('a')
-            self.info['rating'] = rating.strip() + ' (' + votes.string.strip() + ')'
-        else:
-            self.info['rating'] = ''
-
-        runtime = soup.find(text='Runtime:')
-        if runtime and runtime.next:
-            self.info['runtime'] = runtime.next.strip()
-        else:
-            self.info['runtime'] = ''
-
-        # Replace special characters in the items
-        for (k,v) in self.info.items():
-            self.info[k] = self.convert_entities(v)
-
-        if config.DEBUG:
-            for (k,v) in self.info.items():
-                _debug_('items=%s:%s' % (k, v))
-            _debug_('id="%s", dvd="%s"' % (id, dvd))
-            _debug_(self.info, 2)
-
-        # Add impawards.com poster URLs.
-        self.impawardsimages(self.info['title'], self.info['year'])
-
-        # Add images from IMDB database. These images are much smaller than
-        # the impawards ones.
-        if not id:
-            return (self.title, self.info, self.image_urls)
-
-        if not dvd:
-            url = 'http://us.imdb.com/title/tt%s/dvd' % id
-            _debug_('url="%s"' % (url))
-            req = urllib2.Request(url, txdata, txheaders)
-
-            try:
-                r = urllib2.urlopen(req)
-                soup.feed(r.read())
-                r.close()
-                divs = soup.findAll('table', { 'class' : 'dvd_section' })
-                for div in divs:
-                    image = div.find('img')
-                    if image['src'].find('http') < 0:
-                        continue
-                    self.image_urls += [ image['src'] ]
-            except urllib2.HTTPError, error:
-                pass
-            except UnicodeDecodeError:
-                # FIXME:
-                # This is a bad hack. Some character could not be converted to ascii.
-                # We ignore these errors as it does not really affect the FXD output.
-                pass
-        _debug_('image_urls=%s' % (self.image_urls))
-
-        return (self.title, self.info, self.image_urls)
+        _debug_('id=%r' % (id,))
+        return id
 
 
     def impawardsimages(self, title, year):
@@ -897,8 +918,7 @@ class FxdImdb:
         """parser for posters from www.impawards.com. TODO: check for licences
         of each poster and add all posters"""
 
-        path = '%s/posters/%s.jpg' % (path[:path.rfind('/')], \
-                                      path[path.rfind('/')+1:path.rfind('.')])
+        path = '%s/posters/%s.jpg' % (path[:path.rfind('/')], path[path.rfind('/')+1:path.rfind('.')])
         return [ 'http://%s%s' % (host, path) ]
 
 
@@ -924,7 +944,7 @@ class FxdImdb:
             except:
                 pass
         if not self.image_url:
-            print "Image dowloading failed"
+            print "Image downloading failed"
             return
 
         self.image = (self.fxdfile + '.jpg')
@@ -1145,5 +1165,6 @@ def relative_path(filename):
 
     return filename
 
+
 def point_maker(matching):
     return '%s.%s' % (matching.groups()[0], matching.groups()[1])
